from datetime import datetime
from enum import IntEnum
from typing import Optional

from tic.shared.dtos.psc_api_response import PscApiErrorResponse
from tic.shared.utils.date_util import from_iso_8601, to_iso_8601


class ReputationErrorCode(IntEnum):
    # These are http codes for convenience - but can't be counted on.
    # It's possible that there will be more than one error code for a status code.
    OK = 200
    # these are a 400, so this is the best way for me to represent that?
    INVALID_HASH = 400
    INVALID_FORMAT = 1400
    NO_INPUT_PROVIDED = 2400
    HASH_NOT_FOUND = 404
    UNAUTHORIZED = 401
    FORBIDDEN = 403
    PAYLOAD_TOO_LARGE = 413
    TOO_MANY_REQUESTS = 429
    INTERNAL_SERVER_ERROR = 500


class ReputationErrorResponse(PscApiErrorResponse):
    encode_settings = {
        "expires_at": lambda x: None if x is None else to_iso_8601(x),
        "error_code": lambda x: x.name,
    }
    decode_settings = {
        "expires_at": lambda x: None if x is None else from_iso_8601(x),
        "error_code": lambda x: ReputationErrorCode[x],
    }

    encode_names = {"hash_": "hash"}

    decode_names = {"hash": "hash_"}

    def __init__(
        self,
        error_code: ReputationErrorCode = None,
        message: str = None,
        hash_: str = None,
        expires_at: datetime = None,
    ):
        super().__init__(error_code, message)
        self.hash_: str = hash_
        self.expires_at: Optional[datetime] = expires_at
